@@ -1,4 +1,5 @@
 #include "sa-greedy.h"
+#include "gxe-test.h"
 #include <R_ext/Rdynload.h>
 
 void R_init_logicDT(DllInfo *info) {
@@ -14,6 +15,8 @@ void R_init_logicDT(DllInfo *info) {
   R_RegisterCCallable("logicDT", "greedySearch_",  (DL_FUNC) &greedySearch_);
   R_RegisterCCallable("logicDT", "calcAUC_",  (DL_FUNC) &calcAUC_);
   R_RegisterCCallable("logicDT", "fit4plModel_",  (DL_FUNC) &fit4plModel_);
+  R_RegisterCCallable("logicDT", "fit4plModelWithGroups_",  (DL_FUNC) &fit4plModelWithGroups_);
+  R_RegisterCCallable("logicDT", "fitLinearModel_",  (DL_FUNC) &fitLinearModel_);
 
   R_registerRoutines(info, NULL, NULL, NULL, NULL);
   R_useDynamicSymbols(info, TRUE);
